sub readfile($) {
	my $filename = shift;
	local *INFILE;
	open(*INFILE, "< $filename") or die("$filename: $!\n");
	my @content = <INFILE>;
	close(*INFILE);
	return @content;
}

sub writefile($\@) {
	my $filename = shift;
	local *OUTFILE;
	open(*OUTFILE, "> $filename") or die("$filename: $!\n");
	print OUTFILE @{$_[0]};
	close(*OUTFILE);
}

sub addConfigEntries(\@) {
	my @beforefirmware = ();
	my @firmware = ();
	my @afterfirmware = ();
	my @productids = ();
	my @afterids = ();

	my $foundafirmware = undef;
	my $foundrightfirmware = undef;
	my $foundids = undef;

	foreach my $line (@{$_[0]}) {
		if ($line =~ /^\#\s*firmware\s+(.*\Q$firmwarename\E)\s*$/) {
			if ("$1" eq "$firmwarepath") {
				# Commented line with the right driver => uncomment it
				push @firmware, "firmware $1\n";
				$foundrightfirmware = 1;
			}
			else {
				# Commented line without the right driver => unchange it
				push @firmware, "$line";
			}
			$foundafirmware = 1;
		}
		elsif ($line =~ /^\s*firmware\s+(.*\Q$firmwarename\E)\s*$/) {
			if ("$1" eq "$firmwarepath") {
				# Uncommented line with the right driver => unchange it
				push @firmware, "firmware $1\n";
				$foundrightfirmware = 1;
			}
			else {
				# Uncommented line without the right driver => comment it
				push @firmware, "# firmware $1\n";
			}
			$foundafirmware = 1;
		}
		elsif ($line =~ /^\#\s*firmware\s+.*?\s*$/) {
			# Commented line without the right driver => unchange it
			push @firmware, "$line";
			$foundafirmware = 1;
		}
		elsif ($line =~ /^\s*(firmware\s+.*?)\s*$/) {
			# Uncommented line without the right driver => comment it
			push @firmware, "#$1\n";
			$foundafirmware = 1;
		}
		elsif ($line =~ /^\s*\#?\s*(usb\s+\Q$vendorid\E\s+\Q$productid\E)\s*$/) {
			# uncomment or leave unchanged the product id
			my $l = "$1\n";
			if ($foundids) {
				$l = "#$l";
			}
			else {
				my $lastline;
				if ($foundafirmware) {
					$lastline = pop @afterfirmware;
				}
				else {
					$lastline = pop @beforefirmware;
				}
				if ($lastline !~ /^\s*\#\s*Epson\s+Perfection\s+/i) {
					if ($foundafirmware) {
						push @afterfirmware, "$lastline";
					}
					else {
						push @beforefirmware, "$lastline";
					}
				}
				push @productids, "# Epson Perfection $firmwaremodel\n";
				$foundids = 1;
			}
			push @productids, "$l";
		}
		else {
			# Another type of line => unchange it
			if ($foundids) {
				push @afterids, "$line";
			}
			elsif ($foundafirmware) {
				push @afterfirmware, "$line";
			}
			else {
				push @beforefirmware, "$line";
			}
		}
	}

	if (!$foundrightfirmware) {
		push @firmware, "firmware $firmwarepath\n";
	}

	my @content = (@beforefirmware,@firmware,@afterfirmware);

	if (!$foundids) {
		if ((@content)&&($content[$#content] ne "\n")) {
			push @productids, "\n";
		}
		push @productids, "# Epson Perfection $firmwaremodel\nusb $vendorid $productid\n";
	}

	return (@content,@productids,@afterids);
}

sub removeConfigEntries(\@) {
	my @beforefirmware = ();
	my @firmware = ();
	my @afterfirmware = ();

	my $foundafirmware = undef;

	foreach my $line (@{$_[0]}) {
		if ($line =~ /^\#\s*firmware\s+(.*\Q$firmwarename\E)\s*$/) {
			# Commented line with the right driver => remove it
			# Commented line without the right driver => unchange it
			if ("$1" ne "$firmwarepath") {
				push @firmware, "$line";
			}
			$foundafirmware = 1;
		}
		elsif ($line =~ /^\s*firmware\s+(.*\Q$firmwarename\E)\s*$/) {
			# Uncommented line with the right driver => remove it
			# Uncommented line without the right driver => unchange it
			if ("$1" ne "$firmwarepath") {
				push @firmware, "$line";
			}
			$foundafirmware = 1;
		}
		elsif ($line =~ /^\#?\s*firmware\s+[^\s]+\s*$/) {
			# (Un)Commented line without the right driver => unchange it
			push @firmware, "$line";
			$foundafirmware = 1;
		}
		else {
			# Another type of line => unchange it
			if ($foundafirmware) {
				push @afterfirmware, "$line";
			}
			else {
				push @beforefirmware, "$line";
			}
		}
	}

	if (!@firmware) {
		push @firmware, "#firmware path/to/firmware\n";
	}

	return (@beforefirmware,@firmware, @afterfirmware);
}

sub postInstallation($) {
	if ($_[0] eq "configure") {

	#  Enable the Epson backend unconditionally, assuming that people who
	#  install this package want to use it.

		print "Registering Epson $firmwaremodel firmware... ";

		my @filecontent = readfile("$dllfile");

		@filecontent = addConfigEntries(@filecontent);

		writefile("$dllfile",@filecontent);

		print "done.\n";

	}

	exit(0);
}

sub preRemoval($) {
	if (($ARGV[0] eq "remove")||($ARGV[0] eq "deconfigure")) {

	#  Disable the Epson backend unconditionally.
	#
		print "Unregistering EPSON $firmwaremodel firmware... ";

		my @filecontent = readfile("$dllfile");

		@filecontent = removeConfigEntries(@filecontent);

		writefile("$dllfile",@filecontent);

		print "done.\n";
	}

	exit(0);
}
